
# src\file_conversor\config\__init__.py

"""
This module initializes the app configuration modules.
"""

from file_conversor.config.config import Configuration
from file_conversor.config.state import State
from file_conversor.config.log import Log



# src\file_conversor\dependency\abstract_pkg_manager.py

"""
This module provides functionalities for handling external backends.
"""

import os
import platform
import shutil
import subprocess

from rich import print
from typing import Any, Callable, Iterable
from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class AbstractPackageManager:

    def __init__(self,
                 dependencies: dict[str, str],
                 env: list[str | Path] | None = None,
                 ) -> None:
        """
        Initializes package manager.

        :param dependencies: External dependencies to check. Format {``executable: dependency``}.        
        """
        super().__init__()
        self._os_type = platform.system()
        self._dependencies = dependencies
        self._env = env
        self._pre_install_dep_callbacks: list[Callable[[], Any]] = []
        self._post_install_dep_callbacks: list[Callable[[], Any]] = []

    def check_dependencies(self) -> set[str]:
        """
        Check if external dependencies are installed.

        :return: Missing dependencies list.
        """
        missing_dependencies = set([])

        # check if executable exists
        self._set_env_path()
        for executable, dependency in self._dependencies.items():
            found_exe = shutil.which(executable)
            if not found_exe:
                logger.warning(f"ABSTRACT PKG MANAGER - {_('Executable')} '{executable}' {_('not found')}. {_('Marking dependency')} '{dependency}' {_('for installation')}.")
                missing_dependencies.add(dependency)

        # missing dependencies
        return missing_dependencies

    def get_pkg_manager_installed(self) -> str | None:
        """
        Checks if the package manager is already installed in system.

        :return: package manager path if installed, otherwise None

        :raises RuntimeError: if package manager not supported in the OS.
        """
        if self._os_type not in self.get_supported_oses():
            raise RuntimeError(f"{_("Package manager is not supported on")} '{self._os_type}'.")
        return self._get_pkg_manager_installed()

    def get_supported_oses(self) -> set[str]:
        """
        Gets the package manager supported OSes.

        :return: Set([PLATFORM_WINDOWS, PLATFORM_LINUX, PLATFORM_MACOS, ...])
        """
        return self._get_supported_oses()

    def install_pkg_manager(self) -> str | None:
        """
        Installs package manager for the current user.

        :return: package manager path if installation success, otherwise None .

        :raises RuntimeError: if package manager not supported in the OS.
        :raises RuntimeError: package manager already installed in system.
        :raises subprocess.CalledProcessError: package manager could not be installed in system.
        """
        if self.get_pkg_manager_installed():
            raise RuntimeError("Package manager already installed in system.")
        logger.info(f"{_('Installing package manager')} ...")
        logger.debug(f"{self._get_cmd_install_pkg_manager()}")
        subprocess.run(self._get_cmd_install_pkg_manager(), check=True)
        self._post_install_pkg_manager()
        return self.get_pkg_manager_installed()

    def install_dependencies(self, dependencies: Iterable[str]):
        """
        Installs dependencies with package manager.

        :param dependencies: External dependencies to install. Format [``dependency``, ...].

        :raises RuntimeError: package manager NOT installed in system.
        :raises subprocess.CalledProcessError: dependency could not be installed in system.
        """
        if not self.get_pkg_manager_installed():
            raise RuntimeError("Package manager NOT installed in system.")
        logger.info(f"{_('Running pre-install dependency commands')} ...")
        for callback in self._pre_install_dep_callbacks:
            callback()
        logger.info(f"{_('Installing missing dependencies')} {list(dependencies)} ...")
        for dep in dependencies:
            subprocess.run(self._get_cmd_install_dep(dep), check=True)
        logger.info(f"{_('Running post-install dependency commands')} ...")
        for callback in self._post_install_dep_callbacks:
            callback()

    def add_pre_install_callback(self, callback: Callable[[], Any]):
        """ Adds a pre-install callback to run before installing dependencies. """
        self._pre_install_dep_callbacks.append(callback)

    def add_post_install_callback(self, callback: Callable[[], Any]):
        """ Adds a post-install callback to run after installing dependencies. """
        self._post_install_dep_callbacks.append(callback)

    def _set_env_path(self):
        if not self._env:
            logger.debug(f"No env PATH to set for pkg manager")
            return
        # check if needs to add path
        env_paths = os.environ["PATH"].split(os.pathsep)
        for path_str in self._env:
            path_str = str(path_str)
            if path_str in env_paths:
                logger.debug(f"Skipping path '{path_str}'. Already in ENV")
                continue
            env_paths.append(path_str)
        os.environ["PATH"] = os.pathsep.join(env_paths)

    def _get_pkg_manager_installed(self) -> str | None:
        raise NotImplementedError("Method not overloaded.")

    def _get_supported_oses(self) -> set[str]:
        raise NotImplementedError("Method not overloaded.")

    def _get_cmd_install_pkg_manager(self) -> list[str]:
        raise NotImplementedError("Method not overloaded.")

    def _post_install_pkg_manager(self) -> None:
        raise NotImplementedError("Method not overloaded.")

    def _get_cmd_install_dep(self, dependency: str) -> list[str]:
        raise NotImplementedError("Method not overloaded.")



# src\file_conversor\backend\batch_backend.py

"""
This module provides functionalities for barch file processing using this app.
"""

import shlex
import os
import subprocess
import json
import re

from pathlib import Path

from rich import print
from rich.progress import Progress

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.rich import DummyProgress

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class BatchBackend(AbstractBackend):
    """Class to provide batch file processing, using pipelines"""
    SUPPORTED_IN_FORMATS = {}
    SUPPORTED_OUT_FORMATS = {}

    CONFIG_FILENAME = ".config_fc.json"

    def __init__(
        self,
        pipeline_folder: str
    ):
        """
        Initialize the Batch backend.
        """
        super().__init__()

        self._pipeline_path = Path(os.path.expandvars(pipeline_folder)).resolve()
        self._config_file = self._pipeline_path / self.CONFIG_FILENAME

        self._config_data = []

        self._stage_len = 0
        self._stage_num = 1
        self._stage_prev = self._pipeline_path

    def add_stage(self, out_dir: str, command: str):
        stage = f"{self._stage_num}_{out_dir}"
        stage_path = (self._pipeline_path / stage).resolve()
        stage_path.mkdir(exist_ok=True)

        self._config_data.append({
            "in_dir": self._stage_prev,
            "out_dir": stage_path,
            "command": command,
        })
        self._stage_num += 1
        self._stage_prev = stage_path
        logger.info(f"{_('Pipeline stage created at')} '{stage_path}'")

    def save_config(self,):
        self._pipeline_path.mkdir(parents=True, exist_ok=True)
        self._config_file.write_text(json.dumps(self._config_data, indent=2, sort_keys=True))
        logger.info(f"{_('Config file saved at')} '{self._config_file}'")

    def load_config(self,):
        """
        Load configuration file

        :raises RuntimeError: config file does not exist.
        """
        if not self._config_file.exists():
            raise RuntimeError(f"{_('Config file')} '{self._pipeline_path}' {_('does not exist')}")
        logger.info(f"{_('Loading')} '{self._config_file}' ...")
        with open(self._config_file, "r") as f:
            self._config_data = json.load(f)
        self._stage_len = len(self._config_data)
        logger.info(f"{_('Found')} {self._stage_len} {_('stages')}")

    def execute(self, progress: Progress | DummyProgress):
        """
        Executes the batch pipeline

        :raises subprocess.CalledProcessError: if a stage failes.
        """
        task = progress.add_task(f"{_('Processing stage')}", total=self._stage_len)
        for i, stage in enumerate(self._config_data):
            progress.update(task, description=f"{_('Processing stage')} {i + 1}/{self._stage_len}")
            self._execute_stage(progress=progress, **stage)
            progress.update(task, completed=i + 1)

    def _execute_stage(self, progress: Progress | DummyProgress, in_dir: str, out_dir: str, command: str):
        """
        Process a pipeline stage

        :raises subprocess.CalledProcessError: if a stage failes
        """
        input_path = Path(in_dir)
        output_path = Path(out_dir)
        cmd_template = str(command)

        logger.info(f"{_('Executing batch stage')} '{out_dir}' ...")
        total_files = sum(1 for _ in input_path.glob("*"))
        task = progress.add_task(f"{_('Stage')} '{output_path.name}' - {_('Processing files')}", total=total_files)
        for i, in_path in enumerate(input_path.glob("*")):
            # ignore folders and config file
            if in_path.is_dir() or in_path.name == self.CONFIG_FILENAME:
                continue
            # execute cmd
            try:
                cmd_list = self._gen_cmd_list(in_path, in_path=in_path, out_path=output_path, cmd_template=cmd_template)
                logger.debug(f"Command list: '{cmd_list}'")
                process = subprocess.run(cmd_list,
                                         #  cwd=self.SCRIPT_WORKDIR,
                                         capture_output=True,
                                         check=True,
                                         )
                logger.debug(f"Processing file '{in_path}': [bold green]{_('SUCCESS')}[/] ({process.returncode})")
            except Exception as e:
                logger.error(f"Processing file '{in_path}': [bold red]{_('FAILED')}[/]")
                logger.error(f"{str(e)}")
                if isinstance(e, subprocess.CalledProcessError):
                    logger.error(f"Stdout:\n{e.stdout}")
                    logger.error(f"Stderr:\n{e.stderr}")
                self._clean_stage(output_path)
                raise
            finally:
                progress.update(task, completed=i + 1)
        # success, clean input_path
        self._clean_stage(input_path)
        progress.update(task, total=100, completed=100)
        logger.info(f"{_('Finished batch stage')} '{out_dir}'")

    def _clean_stage(self, dir_path: Path):
        """Clean files inside folder"""
        logger.debug(f"Cleaning files from folder '{dir_path}' ...")
        for path in dir_path.glob("*"):
            if path.is_file() and path.name != self.CONFIG_FILENAME:
                path.unlink()

    def _gen_cmd_list(self, input_path: Path, in_path: Path, out_path: Path, cmd_template: str):
        """Creates the command list based on cmd_template"""
        cmd_list = []
        for cmd in shlex.split(f"{State.get_executable()} {cmd_template}"):
            # replace placeholders
            cmd = cmd.replace(f"{{in_file_path}}", f"{input_path.resolve()}")
            cmd = cmd.replace(f"{{in_file_name}}", f"{input_path.with_suffix("").name}")
            cmd = cmd.replace(f"{{in_file_ext}}", f"{input_path.suffix[1:]}")
            cmd = cmd.replace(f"{{in_dir}}", f"{in_path}")
            cmd = cmd.replace(f"{{out_dir}}", f"{out_path}")
            # normalize paths
            if "/" in cmd or "\\" in cmd:
                cmd = os.path.normpath(cmd)
            cmd_list.append(cmd)
        return cmd_list



# src\file_conversor\backend\office\abstract_msoffice_backend.py

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.system import CURR_PLATFORM, PLATFORM_WINDOWS

# conditional import
if CURR_PLATFORM == PLATFORM_WINDOWS:
    import pythoncom
    from win32com import client
else:
    pythoncom = None
    client = None

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class Win32Com:
    def __init__(self, prog_id, visible: bool | None = None):
        """
        Win32Com dispatch wrapper class

        prog_id: The registered COM program name, e.g., 'Word.Application'
        visible: Set to True to make the application visible. Defaults to None.
        """
        super().__init__()
        self._prog_id = prog_id
        self._visible = visible
        self._app = None

    def __enter__(self):
        if not client:
            raise OSError("Win32Com is only available in Windows OS")
        self._app = client.Dispatch(self._prog_id)
        try:
            if self._visible is not None:
                self._app.Visible = self._visible
        except AttributeError:
            pass  # Some COM objects don't have Visible
        return self._app

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            if self._app:
                self._app.Quit()
        except Exception:
            logger.warning(f"Fail to quit Win32Com prog_id '{self._prog_id}'. Try again later.")
        self._app = None
        # Returning False means exceptions (if any) will propagate
        return False


class AbstractMSOfficeBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``msoffice``.
    """

    def __init__(
        self,
        prog_id: str,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose
        self.PROG_ID = prog_id

    def convert(
        self,
        output_file: str,
        input_file: str,
    ):
        """
        Convert input file into an output file.

        :param output_file: Output file.
        :param input_file: Input file.        

        :raises FileNotFoundError: if input file not found.
        :raises OSError: if os is not Windows, or MS Office App not available.
        """
        raise NotImplementedError("Must be implemented within a subclass")

    def is_available(self) -> bool:
        """Returns True if MS Office App is available, False otherwise"""
        if not pythoncom:
            return False
        try:
            pythoncom.CoInitialize()
            clsid = pythoncom.ProgIDFromCLSID(self.PROG_ID)
            return clsid is not None
        except:
            logger.warning(f"Microsoft Office '{self.PROG_ID}' backend not available.")
            return False
        finally:
            pythoncom.CoUninitialize()




# src\file_conversor\cli\audio_video_cmd.py

import subprocess
import time
import typer

from typing import Annotated
from datetime import timedelta
from pathlib import Path

from rich import print
from rich.text import Text
from rich.panel import Panel
from rich.console import Group

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.rich import get_progress_bar
from file_conversor.utils.validators import check_positive_integer, check_file_format
from file_conversor.utils.formatters import format_bitrate, format_bytes

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

audio_video_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = State.get_icons_folder()
    for ext in FFmpegBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="info",
                description="Get Info",
                command=f'cmd /k "{State.get_executable()} audio-video info "%1""',
                icon=str(icons_folder_path / 'info.ico'),
            ),
            WinContextCommand(
                name="to_avi",
                description="To AVI",
                command=f'{State.get_executable()} audio-video convert "%1" -o "%1.avi"',
                icon=str(icons_folder_path / 'avi.ico'),
            ),
            WinContextCommand(
                name="to_mp4",
                description="To MP4",
                command=f'{State.get_executable()} audio-video convert "%1" -o "%1.mp4"',
                icon=str(icons_folder_path / 'mp4.ico'),
            ),
            WinContextCommand(
                name="to_mkv",
                description="To MKV",
                command=f'{State.get_executable()} audio-video convert "%1" -o "%1.mkv"',
                icon=str(icons_folder_path / 'mkv.ico'),
            ),
            WinContextCommand(
                name="to_mp3",
                description="To MP3",
                command=f'{State.get_executable()} audio-video convert "%1" -o "%1.mp3"',
                icon=str(icons_folder_path / 'mp3.ico'),
            ),
            WinContextCommand(
                name="to_m4a",
                description="To M4A",
                command=f'{State.get_executable()} audio-video convert "%1" -o "%1.m4a"',
                icon=str(icons_folder_path / 'm4a.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


# audio_video info
@audio_video_cmd.command(
    help=f"""
        {_('Get information about a audio/video file.')}

        {_('This command retrieves metadata and other information about the audio / video file')}:

        - {_('Format')} (mp3, mp4, mov, etc)

        - {_('Duration')} (HH:MM:SS)

        - Bitrate

        - {_('Other properties')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor audio-video info filename.webm`

        - `file_conversor audio-video info other_filename.mp3`
    """)
def info(
    filename: Annotated[str, typer.Argument(
        help=f"{_('File')} ({', '.join(FFmpegBackend.SUPPORTED_IN_FORMATS)})",
        callback=lambda x: check_file_format(x, FFmpegBackend.SUPPORTED_IN_FORMATS, exists=True),
    )],
):

    formatted = []
    metadata: dict
    with get_progress_bar() as progress:
        ffprobe_task = progress.add_task(f"{_('Parsing file metadata')} ...", total=None)

        ffmpeg_backend = FFmpegBackend(
            install_deps=CONFIG['install-deps'],
            verbose=STATE["verbose"],
        )
        metadata = ffmpeg_backend.get_file_info(filename)
        progress.update(ffprobe_task, total=100, completed=100)

    # 📁 Informações gerais do arquivo
    if "format" in metadata:
        format_info: dict = metadata["format"]

        duration = format_info.get('duration', 'N/A')
        if duration != "N/A":
            duration_secs = int(float(duration))
            duration_td = timedelta(seconds=duration_secs)
            duration = str(duration_td)
        size = format_info.get("size", "N/A")
        if size != "N/A":
            size = format_bytes(float(size))
        bitrate = format_info.get('bit_rate', 'N/A')
        if bitrate != "N/A":
            bitrate = format_bitrate(int(bitrate))

        formatted.append(Text(f"📁 {_('File Information')}:", style="bold cyan"))
        formatted.append(f"  - {_('Name')}: {filename}")
        formatted.append(f"  - {_('Format')}: {format_info.get('format_name', 'N/A')}")
        formatted.append(f"  - {_('Duration')}: {duration}")
        formatted.append(f"  - {_('Size')}: {size}")
        formatted.append(f"  - {_('Bitrate')}: {bitrate}")

    # 🎬 Streams de Mídia
    if "streams" in metadata:
        if len(metadata["streams"]) > 0:
            formatted.append(Text(f"\n🎬 {_("Media Streams")}:", style="bold yellow"))
        for i, stream in enumerate(metadata["streams"]):
            stream_type = stream.get("codec_type", "unknown")
            codec = stream.get("codec_name", "N/A")
            resolution = f"{stream.get('width', '?')}x{stream.get('height', '?')}" if stream_type == "video" else ""
            bitrate = stream.get("bit_rate", "N/A")

            if bitrate != "N/A":
                bitrate = format_bitrate(int(bitrate))

            formatted.append(f"\n  🔹 {_('Stream')} #{i} ({stream_type.upper()}):")
            formatted.append(f"    - {_('Codec')}: {codec}")
            if resolution:
                formatted.append(f"    - {_('Resolution')}: {resolution}")
            formatted.append(f"    - {_('Bitrate')}: {bitrate}")
            if stream_type == "audio":
                formatted.append(f"    - {_('Sampling rate')}: {stream.get('sample_rate', 'N/A')} Hz")
                formatted.append(f"    - {_('Channels')}: {stream.get('channels', 'N/A')}")

    # 📖 Capítulos
    if "chapters" in metadata:
        if len(metadata["chapters"]) > 0:
            formatted.append(Text(f"\n📖 {_('Chapters')}:", style="bold green"))
        for chapter in metadata["chapters"]:
            title = chapter.get('tags', {}).get('title', 'N/A')
            start = chapter.get('start_time', 'N/A')
            formatted.append(f"  - {title} ({_('Time')}: {start}s)")

    # Agrupar e exibir tudo com Rich
    group = Group(*formatted)
    print(Panel(group, title=f"🧾 {_('File Analysis')}", border_style="blue"))


# audio_video convert
@audio_video_cmd.command(
    help=f"""
        {_('Convert a audio/video file to a different format.')}

        {_('This command can be used to convert audio or video files to the specified format.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor audio-video convert input_file.webm -o output_file.mp4 --audio-bitrate 192`

        - `file_conversor audio-video convert input_file.mp4 -o output_file.mp3`
    """)
def convert(
    input_file: Annotated[str, typer.Argument(
        help=f"{_('Input file')} ({', '.join(FFmpegBackend.SUPPORTED_IN_FORMATS)})",
        callback=lambda x: check_file_format(x, FFmpegBackend.SUPPORTED_IN_FORMATS, exists=True),
    )],
    output_file: Annotated[str, typer.Option("--output", "-o",
                                             help=f"{_('Output file')} ({', '.join(FFmpegBackend.SUPPORTED_OUT_FORMATS)})",
                                             callback=lambda x: check_file_format(x, FFmpegBackend.SUPPORTED_OUT_FORMATS),
                                             )],
    audio_bitrate: Annotated[int, typer.Option("--audio-bitrate", "-ab",
                                               help=_("Audio bitrate in kbps"),
                                               callback=check_positive_integer,
                                               )] = CONFIG["audio-bitrate"],

    video_bitrate: Annotated[int, typer.Option("--video-bitrate", "-vb",
                                               help=_("Video bitrate in kbps"),
                                               callback=check_positive_integer,
                                               )] = CONFIG["video-bitrate"],
):
    process: subprocess.Popen | None = None
    in_options = []
    out_options = []

    out_ext = Path(output_file).suffix[1:]

    # configure out options
    out_options.extend(["-b:a", f"{audio_bitrate}k"])
    if out_ext in FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS:
        out_options.extend(["-b:v", f"{video_bitrate}k"])

    # execute ffmpeg
    ffmpeg_backend = FFmpegBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )
    input_file_duration = ffmpeg_backend.calculate_file_total_duration(input_file)
    process = ffmpeg_backend.convert(
        input_file,
        output_file,
        in_options=in_options,
        out_options=out_options,
    )

    # display current progress
    with get_progress_bar() as progress:
        ffmpeg_task = progress.add_task(f"{_('Processing file')} '{input_file}':", total=100)
        while process.poll() is None:
            ffmpeg_completed = FFmpegBackend.get_convert_progress(process, input_file_duration)
            progress.update(ffmpeg_task, completed=ffmpeg_completed)
            time.sleep(0.25)
        progress.update(ffmpeg_task, completed=100)

    process.wait()
    if process.returncode != 0:
        raise RuntimeError(process.stdout)

    logger.info(f"{_('FFMpeg convertion')}: [green][bold]{_('SUCCESS')}[/bold][/green] ({process.returncode})")



# src\file_conversor\backend\pikepdf_backend.py

"""
This module provides functionalities for handling PDF files using ``pikepdf`` backend (qpdf python wrapper).
"""

import pikepdf

from pikepdf import ObjectStreamMode
from typing import Any, Callable

# user-provided imports
from file_conversor.config.log import Log

from file_conversor.backend.abstract_backend import AbstractBackend

LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class PikePDFBackend(AbstractBackend):
    """
    A class that provides an interface for handling PDF files using ``pikepdf`` (qpdf python wrapper).
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {},
    }

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the ``pikepdf`` backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    @staticmethod
    def is_encrypted(file_path: str) -> bool:
        """Checks if PDF file is encrypted"""
        try:
            with pikepdf.open(file_path):
                return False  # opened without password → not encrypted
        except pikepdf.PasswordError:
            return True  # opening failed because it's encrypted
        except Exception as e:
            raise e  # other errors (corrupted file, etc.)

    def repair(
        self,
        input_file: str,
        output_file: str,
        progress_callback: Callable[[int], Any] | None = None,
        decrypt_password: str | None = None,
        compress: bool = True,
    ):
        """
        Repair input PDF file.

        :param input_files: Input PDF file. 
        :param output_files: Output PDF file.
        :param progress_callback: Progress callback executed as PDF is processed. Format callback(0-100). Defaults to None (no progress callback).
        :param decryption_password: Decryption password for input PDF file. Defaults to None (do not decrypt).
        :param compress: Compress PDF output file structures. Defaults to True (compress structures, losslessly).

        :raises FileNotFoundError: if input file not found.
        :raises PDFError, ForeignObjectError: if qpdf errors.
        """
        self.check_file_exists(input_file)

        # Open PDF (with password if provided)
        preserve_encryption = True if decrypt_password and self.is_encrypted(input_file) else None
        with pikepdf.open(input_file, password=decrypt_password if decrypt_password else "") as pdf:
            # Save optimized PDF
            pdf.save(output_file,
                     progress=progress_callback,  # callback(0-100)
                     encryption=preserve_encryption,  # preserve encryption
                     compress_streams=compress,  # compress streams
                     object_stream_mode=ObjectStreamMode(ObjectStreamMode.generate),  # generate streams as needed (max compression)
                     )



# src\file_conversor\backend\office\excel_backend.py

from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.office.abstract_msoffice_backend import AbstractMSOfficeBackend, Win32Com

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class ExcelBackend(AbstractMSOfficeBackend):
    """
    A class that provides an interface for handling doc files using ``excel`` (comtypes).
    """

    SUPPORTED_IN_FORMATS = {
        "xls": {},
        "xlsx": {},
        "ods": {},
    }
    SUPPORTED_OUT_FORMATS = {
        # format = xlFormat VBA code
        # https://learn.microsoft.com/en-us/office/vba/api/excel.xlfileformat
        "xls": {'format': 56},
        "xlsx": {'format': 51},
        "ods": {'format': 60},
        "csv": {'format': 6},
        "pdf": {'format': 57},
        "html": {'format': 44},
    }

    def __init__(
        self,
        install_deps: bool | None = None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Reserved for future use. 
        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(
            prog_id="Excel.Application",
            install_deps=install_deps,
            verbose=verbose,
        )

    def convert(
        self,
        output_file: str,
        input_file: str,
    ):
        """
        Convert input file into an output file.

        :param output_file: Output file.
        :param input_file: Input file.        

        :raises FileNotFoundError: if input file not found.
        :raises RuntimeError: if os != Windows.
        """
        input_path = Path(input_file).resolve()
        output_path = Path(output_file).resolve()

        self.check_file_exists(str(input_path))

        out_config = ExcelBackend.SUPPORTED_OUT_FORMATS[output_path.suffix[1:]]

        with Win32Com(self.PROG_ID, visible=False) as excel:
            workbook = excel.Workbooks.Open(str(input_path))
            if output_path.suffix.lower() == ".pdf":
                workbook.ExportAsFixedFormat(
                    Filename=str(output_path),
                    Type=0,  # 0 = pdf
                    Quality=0,
                    IncludeDocProperties=True,
                    IgnorePrintAreas=False,
                    OpenAfterPublish=False,
                )
            else:
                workbook.SaveAs(
                    str(output_path),
                    FileFormat=out_config['format'],
                )
            workbook.Close(SaveChanges=False)



# src\file_conversor\backend\ghostscript_backend.py

"""
This module provides functionalities for handling files using ``ghostscript`` backend.
"""

import subprocess
from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.dependency import ScoopPackageManager, BrewPackageManager

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class GhostscriptBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``ghostscript``.
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {},
    }

    COMPRESSION_HIGH = "screen"
    """72 dpi quality - high compression / low quality"""

    COMPRESSION_MEDIUM = "ebook"
    """150 dpi quality - medium compression / medium quality"""

    COMPRESSION_LOW = "printer"
    """300 dpi quality - low compression / high quality"""

    COMPRESSION_NONE = "preprint"
    """600 dpi quality - no compression / highest quality"""

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the FFMpeg backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 

        :raises RuntimeError: if ffmpeg dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "gs": "ghostscript"
                }),
                BrewPackageManager({
                    "gs": "ghostscript"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        # find ghostscript bin
        self._ghostscript_bin = self.find_in_path("gs")

    def compress(self,
                 output_file: str,
                 input_file: str,
                 compression_level: str,
                 compatibility_preset: str = "1.5",
                 ):
        """
        Compress input PDF files.

        :param output_file: Output file
        :param input_file: Input file.         
        :param compression_level: Compression level.
        :param compatibility_level: PDF compatibility level (1.3 - 1.7). Defaults to 1.5 (good compatibility / stream compression support).

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if output format is unsupported
        """
        self.check_file_exists(input_file)
        in_path = Path(input_file)
        out_path = Path(output_file)

        if compression_level not in [
            self.COMPRESSION_HIGH,
            self.COMPRESSION_MEDIUM,
            self.COMPRESSION_LOW,
            self.COMPRESSION_NONE,
        ]:
            raise ValueError(
                f"Unsupported compression level: {compression_level}")

        if compatibility_preset not in [
            "1.3",
            "1.4",
            "1.5",
            "1.6",
            "1.7",
        ]:
            raise ValueError(
                f"Unsupported compatibility level: {compatibility_preset}")

        # build command
        command = [
            f"{self._ghostscript_bin}",
            f"-dNOPAUSE",
            f"-dBATCH",
            f"-sDEVICE=pdfwrite",
            f"-dPDFSETTINGS=/{compression_level}",  # compression settings
            f"-dCompatibilityLevel={compatibility_preset}",  # PDF compatibility
            f"-sOutputFile={out_path}",
            f"{in_path}",
        ]

        logger.info(f"Executing Ghostscript ...")
        logger.debug(f"{" ".join(command)}")

        # Execute the FFmpeg command
        _convert_process = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True
        )
        return _convert_process



# src\file_conversor\config\log.py

import re
import logging

from rich import print

from logging import Handler
from logging.handlers import TimedRotatingFileHandler

from types import TracebackType
from typing import Mapping

from pathlib import Path


class Log:
    class CustomLogger:
        def __init__(self, name: str | None) -> None:
            super().__init__()
            self._name = name
            self._log_to_file = True

        @property
        def _logger(self):
            return logging.getLogger(self._name)

        @property
        def level(self) -> int:
            if self._logger.level > logging.NOTSET:
                return self._logger.level
            return logging.getLogger().level

        @property
        def log_to_file(self) -> bool:
            return self._log_to_file

        @log_to_file.setter
        def log_to_file(self, value: bool):
            self._log_to_file = value

        def critical(self, msg: object, *args: object, exc_info: None | bool | tuple[type[BaseException], BaseException, TracebackType | None] | tuple[None, None, None] | BaseException = None, stack_info: bool = False, stacklevel: int = 1, extra: Mapping[str, object] | None = None) -> None:
            if self.log_to_file:
                self._logger.critical(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)
            if self.level <= logging.CRITICAL:
                print(f"[bold reverse red][CRITICAL][/]: {msg}")

        def fatal(self, msg: object, *args: object, exc_info: None | bool | tuple[type[BaseException], BaseException, TracebackType | None] | tuple[None, None, None] | BaseException = None, stack_info: bool = False, stacklevel: int = 1, extra: Mapping[str, object] | None = None) -> None:
            if self.log_to_file:
                self._logger.fatal(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)
            if self.level <= logging.FATAL:
                print(f"[bold reverse red][FATAL][/]: {msg}")

        def error(self, msg: object, *args: object, exc_info: None | bool | tuple[type[BaseException], BaseException, TracebackType | None] | tuple[None, None, None] | BaseException = None, stack_info: bool = False, stacklevel: int = 1, extra: Mapping[str, object] | None = None) -> None:
            if self.log_to_file:
                self._logger.error(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)
            if self.level <= logging.ERROR:
                print(f"[bold red][ERROR][/]: {msg}")

        def warning(self, msg: object, *args: object, exc_info: None | bool | tuple[type[BaseException], BaseException, TracebackType | None] | tuple[None, None, None] | BaseException = None, stack_info: bool = False, stacklevel: int = 1, extra: Mapping[str, object] | None = None) -> None:
            if self.log_to_file:
                self._logger.warning(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)
            if self.level <= logging.WARNING:
                print(f"[bold yellow][WARN][/]: {msg}")

        def info(self, msg: object, *args: object, exc_info: None | bool | tuple[type[BaseException], BaseException, TracebackType | None] | tuple[None, None, None] | BaseException = None, stack_info: bool = False, stacklevel: int = 1, extra: Mapping[str, object] | None = None) -> None:
            if self.log_to_file:
                self._logger.info(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)
            if self.level <= logging.INFO:
                print(f"[bold white][INFO][/]: {msg}")

        def debug(self, msg: object, *args: object, exc_info: None | bool | tuple[type[BaseException], BaseException, TracebackType | None] | tuple[None, None, None] | BaseException = None, stack_info: bool = False, stacklevel: int = 1, extra: Mapping[str, object] | None = None) -> None:
            if self.log_to_file:
                self._logger.debug(msg, *args, exc_info=exc_info, stack_info=stack_info, stacklevel=stacklevel, extra=extra)
            if self.level <= logging.DEBUG:
                print(f"[bold cyan][DEBUG][/]: {msg}")

    class StripMarkupFormatter(logging.Formatter):
        # Use a custom formatter that strips Rich markup
        TAG_RE = re.compile(r'\[/?[^\]]+\]')  # matches [tag] and [/tag]

        def format(self, record):
            if isinstance(record.msg, str):
                record.msg = self.TAG_RE.sub('', record.msg)
            return super().format(record)

    # singleton instance
    __instance = None

    # most severe level, to least
    LEVEL_CRITICAL, LEVEL_FATAL = logging.CRITICAL, logging.FATAL  # 50
    LEVEL_ERROR = logging.ERROR  # 40
    LEVEL_WARNING = logging.WARNING  # 30
    LEVEL_INFO = logging.INFO  # 20
    LEVEL_DEBUG = logging.DEBUG  # 10

    # logfile name
    FILENAME = f".file_conversor.log"

    @staticmethod
    def get_instance(dest_folder: str | Path | None = ".", level: int = LEVEL_INFO):
        """
        Initialize logfile instance

        :param dest_folder: Destination folder to store log file. If None, do not log to file. Defaults to '.' (log to current working folder).
        :param level: Log level. Defaults to LEVEL_INFO.
        """
        if not Log.__instance:
            Log.__instance = Log(dest_folder=dest_folder, level=level)
        return Log.__instance

    def __init__(self, dest_folder: str | Path | None, level: int) -> None:
        """
        Initialize logfile, inside a dest_folder with a log_level
        """
        super().__init__()
        self._dest_path: Path | None = None
        self._file_handler: Handler | None = None

        # configure logger
        self._log_formatter = Log.StripMarkupFormatter(
            fmt='[%(asctime)s] - [%(levelname)s]: %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )

        # set level
        self.set_level(level)
        self.set_dest_folder(dest_folder)

    def getLogger(self, name: str | None = None) -> CustomLogger:
        return Log.CustomLogger(name)

    def get_level(self) -> int:
        return logging.getLogger().level

    def set_level(self, level: int):
        logging.getLogger().setLevel(level)

    def get_dest_folder(self) -> Path | None:
        return self._dest_path

    def set_dest_folder(self, dest_folder: str | Path | None):
        """Activates / deactivates file logging, and sets destination folder"""
        if not dest_folder:
            self._remove_handler(self._file_handler)
            self._dest_path = None
            self._file_handler = None
            return

        self._dest_path = Path(dest_folder).resolve()

        self._file_handler = TimedRotatingFileHandler(
            filename=(self._dest_path / Log.FILENAME).resolve(),
            when='midnight',     # rotate at midnight
            interval=1,          # every 1 day
            backupCount=7,       # keep 7 days of logs
            encoding='utf-8',
            utc=False            # set to True if you want UTC-based rotation
        )
        self._add_handler(self._file_handler)

    def _remove_handler(self, handler: Handler | None):
        if handler:
            logging.getLogger().removeHandler(handler)

    def _add_handler(self, handler: Handler):
        handler.setFormatter(self._log_formatter)
        logging.getLogger().addHandler(handler)



# src\file_conversor\backend\__init__.py

"""
Initialization module for the backend package.

This module imports all functionalities from backend wrappers,
making them available when importing the backend package.
"""

# LIBREOFFICE / MSOFFICE
from file_conversor.backend.office import DOC_BACKEND, XLS_BACKEND, PPT_BACKEND

# PDF
from file_conversor.backend.pdf import *

# OTHER BACKENDS
from file_conversor.backend.batch_backend import BatchBackend
from file_conversor.backend.ffmpeg_backend import FFmpegBackend

from file_conversor.backend.img2pdf_backend import Img2PDFBackend
from file_conversor.backend.pillow_backend import PillowBackend

from file_conversor.backend.pymusvg_backend import PyMuSVGBackend

from file_conversor.backend.win_reg_backend import WinRegBackend




# src\file_conversor\cli\batch_cmd.py

import typer

from typing import Annotated

from rich import print

# user-provided modules
from file_conversor.backend.batch_backend import BatchBackend

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.rich import get_progress_bar

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

batch_cmd = typer.Typer()


# batch create
@batch_cmd.command(
    help=f"""
        {_('Creates a batch file processing pipeline (for tasks automation).')}        

        {_('Will ask questions interactively to create a batch file processing pipeline.')}

        [bold]{_('Placeholders available for commands')}[/]:

        - [bold]{{in_file_path}}[/]: {_('Replaced by the first file path found in pipeline stage.')}

            - Ex: C:/Users/Alice/Desktop/pipeline_name/my_file.jpg

        - [bold]{{in_file_name}}[/]: {_('The name of the input file.')}

            - Ex: my_file

        - [bold]{{in_file_ext}}[/]: {_('The extension of the input file.')}

            - Ex: jpg

        - [bold]{{in_dir}}[/]: {_('The directory of the input path (previous pipeline stage).')}

            - Ex: C:/Users/Alice/Desktop/pipeline_name

        - [bold]{{out_dir}}[/]: {_('The directory of the output path (current pipeline stage).')}

            - Ex: C:/Users/Alice/Desktop/pipeline_name/1_to_png
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor batch create` 
""")
def create():
    logger.info(f"{_('Creating batch pipeline')} ...")
    pipeline_folder: str = typer.prompt(f"{_('Name of the batch pipeline folder (e.g., %USERPROFILE%/Desktop/pipeline_name_here)')}")
    batch_backend = BatchBackend(pipeline_folder)

    terminate = False
    while not terminate:
        try:
            stage: str = typer.prompt(f"{_('Name of the processing stage (e.g., image_convert)')}")

            cmd_str: str = typer.prompt(f"{_('Type command here')} ({_('e.g.')}, image convert {{in_file_path}} {{out_dir}}/{{in_file_name}}_converted.png )")
            batch_backend.add_stage(stage, command=cmd_str)

            terminate = not typer.confirm(f"{_('Need another pipeline stage')}", default=False)
            print(f"-------------------------------------")
        except (KeyboardInterrupt, typer.Abort) as e:
            terminate = True
            raise
        except Exception as e:
            logger.error(f"{str(e)}")

    batch_backend.save_config()
    logger.info(f"{_('Batch creation')}: [bold green]{_('SUCCESS')}[/].")


# batch execute
@batch_cmd.command(
    help=f"""
        {_('Execute batch file processing pipeline.')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor batch execute c:/Users/Alice/Desktop/pipeline_name` 
""")
def execute(
    pipeline_folder: Annotated[str, typer.Argument(
        help=f"{_('Pipeline folder')}",
    )],
):
    logger.info("Executing batch pipeline ...")
    with get_progress_bar() as progress:
        batch_backend = BatchBackend(pipeline_folder)
        batch_backend.load_config()
        batch_backend.execute(progress)

    logger.info(f"{_('Batch execution')}: [bold green]{_('SUCCESS')}[/].")
    logger.info(f"--------------------------------")




# src\file_conversor\dependency\brew_pkg_manager.py

import os
import shutil

from pathlib import Path

# user-provided imports
from file_conversor.system import PLATFORM_LINUX, PLATFORM_MACOS

from file_conversor.config.locale import get_translation
from file_conversor.dependency.abstract_pkg_manager import AbstractPackageManager

_ = get_translation()


class BrewPackageManager(AbstractPackageManager):
    def __init__(self,
                 dependencies: dict[str, str],
                 env: list[str | Path] | None = None,
                 ) -> None:
        super().__init__(
            dependencies=dependencies,
            env=env,
        )

    def _get_pkg_manager_installed(self):
        return shutil.which("brew")

    def _get_supported_oses(self) -> set[str]:
        return {PLATFORM_LINUX, PLATFORM_MACOS}

    def _get_cmd_install_pkg_manager(self) -> list[str]:
        return ['/bin/bash', '-c', '$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)']

    def _post_install_pkg_manager(self) -> None:
        possible_paths = [
            "/opt/homebrew/bin",               # macOS (Apple Silicon)
            "/usr/local/bin",                  # macOS (Intel)
            os.path.expanduser("~/.linuxbrew/bin"),  # Linux (Homebrew)
        ]

        for path in possible_paths:
            brew_path = shutil.which("brew", path=os.pathsep.join([path]))
            if brew_path:
                os.environ["PATH"] += os.pathsep + path
                break

    def _get_cmd_install_dep(self, dependency: str) -> list[str]:
        pkg_mgr_bin = self._get_pkg_manager_installed()
        pkg_mgr_bin = pkg_mgr_bin if pkg_mgr_bin else "BREW_NOT_FOUND"
        return [pkg_mgr_bin, "install", dependency]



"""
Module for LibreOffice backend (calc, writer, etc)
"""

from file_conversor.backend.office.calc_backend import LibreofficeCalcBackend
from file_conversor.backend.office.excel_backend import ExcelBackend

from file_conversor.backend.office.impress_backend import LibreofficeImpressBackend
from file_conversor.backend.office.powerpoint_backend import PowerPointBackend

from file_conversor.backend.office.writer_backend import LibreofficeWriterBackend
from file_conversor.backend.office.word_backend import WordBackend

DOC_BACKEND = WordBackend if WordBackend().is_available() else LibreofficeWriterBackend
XLS_BACKEND = ExcelBackend if ExcelBackend().is_available() else LibreofficeCalcBackend
PPT_BACKEND = PowerPointBackend if PowerPointBackend().is_available() else LibreofficeImpressBackend



# src\file_conversor\backend\office\abstract_libreoffice_backend.py

import subprocess

from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.dependency.brew_pkg_manager import BrewPackageManager
from file_conversor.dependency.scoop_pkg_manager import ScoopPackageManager

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class AbstractLibreofficeBackend(AbstractBackend):
    """
    A class that provides an interface for handling files using ``libreoffice``.
    """

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "soffice": "libreoffice"
                }, buckets=[
                    "extras",
                ], env=[
                    r"C:\Users\Andre\scoop\apps\libreoffice\current\LibreOffice\program"
                ]),
                BrewPackageManager({
                    "soffice": "libreoffice"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        self._libreoffice_bin = self.find_in_path("soffice")

    def convert(
        self,
        output_file: str,
        input_file: str,
    ):
        """
        Convert input file into an output file.

        :param output_file: Output file.
        :param input_file: Input file.        

        :raises FileNotFoundError: if input file not found.
        :raises RuntimeError: if LibreOffice fails.
        """
        self.check_file_exists(input_file)

        input_path = Path(input_file).resolve()
        output_path = Path(output_file).resolve()

        output_dir = output_path.parent
        output_format = output_path.suffix.lstrip(".").lower()

        command = [
            str(self._libreoffice_bin),
            "--headless",
            "--convert-to",
            str(output_format),
            "--outdir",
            str(output_dir),
            str(input_path)
        ]

        # Execute command
        logger.info("Executing LibreOffice ...")
        logger.debug(" ".join(command))
        process = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
        )
        process.wait()
        if process.returncode != 0:
            raise RuntimeError(
                f"{_('LibreOffice conversion failed with error code')} '{process.returncode}': {process.stderr}"
            )




# src\file_conversor\cli\doc_cmd.py

import typer

from pathlib import Path
from typing import Annotated, List

from rich import print

# user-provided modules
from file_conversor.backend import DOC_BACKEND

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.rich import get_progress_bar
from file_conversor.utils.validators import check_file_format

from file_conversor.system import CURR_PLATFORM, PLATFORM_WINDOWS
from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

# typer PANELS
doc_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = State.get_icons_folder()
    # WordBackend commands
    for ext in DOC_BACKEND.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name=f"to_{ext}",
                description=f"To {ext.upper()}",
                command=f'{State.get_executable()} doc convert "%1" -o "%1.{ext}"',
                icon=str(icons_folder_path / f"{ext}.ico"),
            ) for ext in DOC_BACKEND.SUPPORTED_OUT_FORMATS
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


# doc to-pdf
@doc_cmd.command(
    help=f"""
        {_('Convert document files into other formats (requires Microsoft Word / LibreOffice).')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor doc convert input_file.odt -o output_file.doc`

        - `file_conversor doc convert input_file.docx -o output_file.pdf`
    """)
def convert(
    input_file: Annotated[str, typer.Argument(help=f"{_('Input file')} ({', '.join(DOC_BACKEND.SUPPORTED_IN_FORMATS)})",
                                              callback=lambda x: check_file_format(x, DOC_BACKEND.SUPPORTED_IN_FORMATS, exists=True),
                                              )],

    output_file: Annotated[str, typer.Option("--output", "-o",
                                             help=f"{_('Output file')} ({', '.join(DOC_BACKEND.SUPPORTED_OUT_FORMATS)}). {_('Defaults to None')} ({_('use the same input file as output name')}).",
                                             callback=lambda x: check_file_format(x, DOC_BACKEND.SUPPORTED_OUT_FORMATS),
                                             )],
):
    doc_backend = DOC_BACKEND(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )

    logger.info(f"{_('Processing file')} '{input_file}' => '{output_file}' ...")

    with get_progress_bar() as progress:
        task = progress.add_task(f"{_('Processing file')} '{input_file}':", total=None)
        doc_backend.convert(
            input_file=input_file,
            output_file=output_file,
        )
        progress.update(task, total=100, completed=100)

    logger.info(f"{_('File conversion')}: [green][bold]{_('SUCCESS')}[/bold][/green]")




# src\file_conversor\config\locale.py

import gettext  # app translations / locales
import locale

from file_conversor.config.state import State
from file_conversor.config.config import Configuration

CONFIG = Configuration.get_instance()


# Get translations
def get_system_locale():
    """Get system default locale"""
    lang, _ = locale.getlocale()
    return lang


def get_translation():
    """
    Get translation mechanism, based on user preferences.
    """
    sys_lang = get_system_locale()
    translation = gettext.translation(
        'messages', State.get_locales_folder(),
        languages=[
            CONFIG["language"],
            sys_lang if sys_lang else "en_US",
            "en_US",  # fallback
        ],
        fallback=False
    )
    return translation.gettext



# src\file_conversor\system\win\context_menu.py

from typing import Self, Callable
from pathlib import Path

from rich import print

# user-provided modules
from file_conversor.system.win.reg import WinRegFile, WinRegKey
from file_conversor.system.win.utils import is_admin

from file_conversor.config import State, Log
from file_conversor.config.locale import get_translation

# get app config
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class WinContextCommand:
    def __init__(self, name: str, description: str, command: str, multi_select: bool = True, icon: str | None = None) -> None:
        """
        Creates a windows context menu

        multi_select =
        ```python
        True  # command accepts multi file selection
        False # command available for single files (only)
        ```

        :param name: Name used to create command keys
        :param description: Description of the command (as the user sees it)
        :param command: Command to execute (accepts "%1" for single path input, %* for many inputs - no DOUBLE QUOTES)
        :param multi_select_model: If command valid for multiple files.
        :param icon: Icon to display with command.
        """
        super().__init__()
        self.name = name
        self.description = description
        self.command = command
        self.multi_select_model = "Document" if multi_select else "Single"
        self.icon = icon


class WinContextMenu:
    _instance = None

    @staticmethod
    def get_instance():
        if not WinContextMenu._instance:
            WinContextMenu._instance = WinContextMenu()
        return WinContextMenu._instance

    def __init__(self) -> None:
        """Set context menu for all users, or for current user ONLY"""
        super().__init__()

        self.MENU_NAME = "File Conversor"
        self.ICON_FILE_PATH = Path(f"{State.get_icons_folder()}/icon.ico").resolve()

        self.ROOT_KEY_USER = rf"HKEY_CURRENT_USER\Software\Classes\SystemFileAssociations\{{ext}}\shell\FileConversor"
        self.ROOT_KEY_MACHINE = rf"HKEY_LOCAL_MACHINE\Software\Classes\SystemFileAssociations\{{ext}}\shell\FileConversor"

        self._root_key_template = self.ROOT_KEY_MACHINE if is_admin() else self.ROOT_KEY_USER
        self._reg_file = WinRegFile()
        self._register_callbacks: list[Callable[[Self], None]] = []

    def get_reg_file(self) -> WinRegFile:
        # run callback prior to getting reg_file
        while self._register_callbacks:
            callback = self._register_callbacks.pop()
            callback(self)
        return self._reg_file

    def register_callback(self, function: Callable[[Self], None]) -> None:
        self._register_callbacks.append(function)

    def add_extension(self, ext: str, commands: list[WinContextCommand]):
        """
        Add extension and context menu for commands

        :param ext: Extension. Format .EXT
        :param commands: Format {name: command}
        """
        root_key_name = self._root_key_template.replace(f"{{ext}}", f"{ext}")
        root_key = WinRegKey(root_key_name).update({
            "MUIVerb": self.MENU_NAME,
            "Icon": str(self.ICON_FILE_PATH),
            "SubCommands": "",
        })
        self._reg_file.add_key(root_key)
        for cmd in commands:
            self._reg_file.update([
                WinRegKey(rf"{root_key}\shell\{cmd.name}").update({
                    "MUIVerb": cmd.description,
                    "Icon": cmd.icon if cmd.icon else "",
                    "MultiSelectModel": cmd.multi_select_model,
                }),
                WinRegKey(rf"{root_key}\shell\{cmd.name}\command").update({
                    "@": cmd.command,
                }),
            ])
        logger.debug(f"Added commands for '{root_key}'")



# src\file_conversor\system\__init__.py

"""Stores platform specific methods"""

import platform

PLATFORM_WINDOWS = "Windows"
PLATFORM_LINUX = "Linux"
PLATFORM_MACOS = "Darwin"
PLATFORM_UNKNOWN = ""

CURR_PLATFORM = platform.system()

# dynamically load modules, as needed
if CURR_PLATFORM == PLATFORM_WINDOWS:
    # WINDOWS OS
    from file_conversor.system.win import reload_user_path, is_admin

elif CURR_PLATFORM == PLATFORM_LINUX:
    # LINUX OS
    from file_conversor.system.lin import reload_user_path, is_admin

elif CURR_PLATFORM == PLATFORM_MACOS:
    # MACOS OS
    from file_conversor.system.mac import reload_user_path, is_admin

else:
    # UNKNOWN OS
    CURR_PLATFORM = PLATFORM_UNKNOWN
    from file_conversor.system.dummy import reload_user_path, is_admin




# src\file_conversor\cli\config_cmd.py

import typer

from typing import Annotated

from rich import print
from rich.pretty import Pretty

# user-provided modules
from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.validators import check_is_bool_or_none, check_positive_integer, check_valid_options

# app configuration
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

# create command
config_cmd = typer.Typer()


# config show
@config_cmd.command(help=f"""
    {_('Show the current configuration of the application')}.
""")
def show():
    print(f"{_('Configuration')}:", Pretty(CONFIG.to_dict(), expand_all=True))


# config set
@config_cmd.command(help=f"""
    {_('Configure the default options for the file converter.')}

    **{_('Example')}:** `file_conversor configure --video-bitrate 5000`
    **{_('Example')}:** `file_conversor configure --audio-bitrate 128`
""")
def set(
    language: Annotated[str, typer.Option("--language", "-l",
                                          help=_("Set preferred language for app (if available). Format lang_COUNTRY. Defaults to system preffered language or 'en_US' (English - United States)."),
                                          )] = CONFIG["language"],

    install_deps: Annotated[str | None, typer.Option("--install-deps", "-install",
                                                     help=_("Install missing external dependencies action. 'True' for auto install. 'False' to not install missing dependencies. 'None' to ask user for action."),
                                                     callback=check_is_bool_or_none,
                                                     )] = CONFIG["install-deps"],

    audio_bitrate: Annotated[int, typer.Option("--audio-bitrate", "-ab",
                                               help=_("Audio bitrate in kbps"),
                                               callback=check_positive_integer,
                                               )] = CONFIG["audio-bitrate"],

    video_bitrate: Annotated[int, typer.Option("--video-bitrate", "-vb",
                                               help=_("Video bitrate in kbps"),
                                               callback=check_positive_integer,
                                               )] = CONFIG["video-bitrate"],

    image_quality: Annotated[int, typer.Option("--image-quality", "-iq",
                                               help=_("Image quality (for ``image convert`` command). Valid values are between 1-100."),
                                               min=1, max=100,
                                               )] = CONFIG["image-quality"],

    image_dpi: Annotated[int, typer.Option("--image-dpi", "-id",
                                           help=_("Image quality in dots per inch (DPI) (for ``image to_pdf`` command). Valid values are between 40-3600."),
                                           min=40, max=3600,
                                           )] = CONFIG["image-dpi"],

    image_fit: Annotated[str, typer.Option("--image-fit", "-if",
                                           help=_("Image fit (for ``image to_pdf`` command). Valid only if ``--page-size`` is defined. Valid values are 'into', or 'fill'. Defaults to 'into'. "),
                                           callback=lambda x: check_valid_options(x.lower(), ['into', 'fill']),
                                           )] = CONFIG["image-fit"],

    image_page_size: Annotated[str | None, typer.Option("--image-page-size", "-ip",
                                                        help=_("Page size (for ``image to_pdf`` command). Format (width, height). Other valid values are: 'a4_portrait', 'a4_landscape'. Defaults to None (PDF size = image size)."),
                                                        callback=lambda x: check_valid_options(x.lower() if x else None, [None, 'a4', 'a4_landscape']),
                                                        )] = CONFIG["image-page-size"],

    image_set_metadata: Annotated[bool, typer.Option("--image-set-metadata", "-is",
                                                     help=_("Set PDF metadata (for ``image to_pdf`` command). Defaults to True (set creator, producer, modification date, etc)."),
                                                     callback=check_is_bool_or_none,
                                                     is_flag=True,
                                                     )] = CONFIG["image-set-metadata"],

    pdf_compression: Annotated[str, typer.Option("--pdf-compression", "-pc",
                                                 help=f"{_('Compression level (high compression = low quality). Valid values are')} {', '.join(["low", "medium", "high", "none"])}. {_('Defaults to')} {CONFIG["pdf-compression"]}.",
                                                 callback=lambda x: check_valid_options(x, ["low", "medium", "high", "none"]),
                                                 )] = CONFIG["pdf-compression"],

    pdf_preset: Annotated[str, typer.Option("--pdf-preset", "-pp",
                                            help=f"{_('Compatibility preset. Valid values are')} '1.3', '1.4', ..., '1.7' . {_('Defaults to')} {CONFIG["pdf-preset"]}.",
                                            callback=lambda x: check_valid_options(x, ["1.3", "1.4", "1.5", "1.6", "1.7"]),
                                            )] = CONFIG["pdf-preset"],
):
    # update the configuration dictionary
    CONFIG.update({
        "language": language,
        "install-deps": None if install_deps == "None" or install_deps is None else bool(install_deps),
        "audio-bitrate": audio_bitrate,
        "video-bitrate": video_bitrate,
        "image-quality": image_quality,
        "image-dpi": image_dpi,
        "image-fit": image_fit,
        "image-page-size": image_page_size,
        "image-set-metadata": image_set_metadata,
        "pdf-compression": pdf_compression,
        "pdf-preset": pdf_preset,
    })
    CONFIG.save()
    show()
    logger.info(f"{_('Configuration file')} {_('updated')}.")



# src\file_conversor\system\dummy\utils.py

def is_admin() -> bool:
    """True if app running with admin priviledges, False otherwise."""
    return False


def reload_user_path():
    """Reload user PATH in current process."""
    # dummy method
    pass



# src\file_conversor\backend\pymusvg_backend.py

"""
This module provides functionalities for handling SVG files using ``pymupdf`` backend.
"""

import fitz  # pymupdf
from pathlib import Path

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend


class PyMuSVGBackend(AbstractBackend):
    """
    A class that provides an interface for handling SVG files using ``pymupdf``.
    """

    SUPPORTED_IN_FORMATS = {
        "svg": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "png": {},
        "jpg": {},
    }

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def convert(self,
                output_file: str,
                input_file: str,
                dpi: int = 200,
                ):
        """
        Convert input image files into output file.

        :param output_file: Output file
        :param input_file: Input file. 
        :param dpi: DPI for rendering images. Defaults to 200.

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if output format is unsupported
        """
        self.check_file_exists(input_file)
        in_path = Path(input_file)
        out_path = Path(output_file)

        # open file
        doc = fitz.open(input_file)

        # => .png, .jpg OUTPUT
        for page in doc:
            pix = page.get_pixmap(dpi=dpi)  # type: ignore
            pix.save(f"{out_path.with_suffix("")}{out_path.suffix}")  # type: ignore



# src\file_conversor\backend\office\word_backend.py

from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.office.abstract_msoffice_backend import AbstractMSOfficeBackend, Win32Com

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class WordBackend(AbstractMSOfficeBackend):
    """
    A class that provides an interface for handling doc files using ``word`` (comtypes).
    """

    SUPPORTED_IN_FORMATS = {
        "doc": {},
        "docx": {},
        "odt": {},
    }
    SUPPORTED_OUT_FORMATS = {
        # format = wdFormat VBA code
        # https://learn.microsoft.com/en-us/office/vba/api/word.wdsaveformat
        "doc": {'format': 0},
        "docx": {'format': 16},
        "odt": {'format': 23},
        "pdf": {'format': 17},
        "html": {'format': 8},
    }

    def __init__(
        self,
        install_deps: bool | None = None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Reserved for future use. Defaults to None. 
        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(
            prog_id="Word.Application",
            install_deps=install_deps,
            verbose=verbose,
        )

    def convert(
        self,
        output_file: str,
        input_file: str,
    ):
        input_path = Path(input_file).resolve()
        output_path = Path(output_file).resolve()

        self.check_file_exists(str(input_path))

        out_config = WordBackend.SUPPORTED_OUT_FORMATS[output_path.suffix[1:]]

        with Win32Com(self.PROG_ID, visible=None) as word:
            doc = word.Documents.Open(str(input_path))
            doc.SaveAs(
                str(output_path),
                FileFormat=out_config['format'],
            )
            doc.Close()



# src\file_conversor\cli\app_cmd.py

import sys
import typer

from rich import print

from typing import Annotated

# user-provided imports
from file_conversor.cli import audio_video_cmd
from file_conversor.cli import batch_cmd
from file_conversor.cli import config_cmd
from file_conversor.cli import image_cmd
from file_conversor.cli import pdf_cmd
from file_conversor.cli import svg_cmd
from file_conversor.cli import win_cmd

# office CLI
from file_conversor.cli.office import doc_cmd, ppt_cmd, xls_cmd

from file_conversor.system import CURR_PLATFORM, PLATFORM_WINDOWS

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()

# Create a Typer CLI application
app_cmd = typer.Typer(
    rich_markup_mode="markdown",
    no_args_is_help=True,
    context_settings={
        "help_option_names": ["-h", "--help"],
    }
)

# PANELS
UTILS_CONFIG_PANEL = _("Utils and Config")
MULTIMEDIA_PANEL = _("Multimedia files")
OFFICE_PANEL = _("Office files")

###############
# MULTIMEDIA PANEL
###############

app_cmd.add_typer(audio_video_cmd,
                  name="audio-video",
                  help=_("Audio / Video file manipulation (requires FFMpeg external library)"),
                  rich_help_panel=MULTIMEDIA_PANEL)

app_cmd.add_typer(image_cmd,
                  name="image",
                  help=_("Image file manipulation"),
                  rich_help_panel=MULTIMEDIA_PANEL)

app_cmd.add_typer(svg_cmd,
                  name="svg",
                  help=_("SVG file manipulation"),
                  rich_help_panel=MULTIMEDIA_PANEL)

app_cmd.add_typer(pdf_cmd,
                  name="pdf",
                  help=_("PDF file manipulation"),
                  rich_help_panel=MULTIMEDIA_PANEL)

###############
# OFFICE PANEL
###############

app_cmd.add_typer(doc_cmd,
                  name="doc",
                  help=f"{_('Word file manipulation')} {_('(requires MS Office / LibreOffice)')})",
                  rich_help_panel=OFFICE_PANEL)

app_cmd.add_typer(xls_cmd,
                  name="xls",
                  help=f"{_('Excel file manipulation')} {_('(requires MS Office / LibreOffice)')})",
                  rich_help_panel=OFFICE_PANEL)

app_cmd.add_typer(ppt_cmd,
                  name="ppt",
                  help=f"{_('PowerPoint file manipulation')} {_('(requires MS Office / LibreOffice)')})",
                  rich_help_panel=OFFICE_PANEL)

######################
# UTILS/CONFIG PANEL
######################

if CURR_PLATFORM == PLATFORM_WINDOWS:
    app_cmd.add_typer(win_cmd,
                      name="win",
                      help=_("Windows OS commands (for Windows ONLY)"),
                      rich_help_panel=UTILS_CONFIG_PANEL)

app_cmd.add_typer(batch_cmd,
                  name="batch",
                  help=f"""{_('Batch file processing (task automation)')}

{_('The batch processing pipeline works by monitoring an input folder, passing those files to the next pipeline stage, and processing them inside that stage. This process continues (output of the current stage is the input of the next stage), until those files reach the end of the pipeline.')}



{_('Example')}:

- {_('Input folder')} => {_('Stage 1')} => {_('Stage 2')} => ... => {_('Output Folder')}
""",
    rich_help_panel=UTILS_CONFIG_PANEL)

app_cmd.add_typer(config_cmd,
                  name="config",
                  help=_("Configure default options"),
                  rich_help_panel=UTILS_CONFIG_PANEL)


#####################
#     APP PANEL
#####################


# help
@app_cmd.command(
    help=f"{_('Show the application help')}",
    rich_help_panel=UTILS_CONFIG_PANEL)
def help():
    ctx = typer.Context(typer.main.get_command(app_cmd))
    print(ctx.command.get_help(ctx))


# Main callback, to process global options
@app_cmd.callback(
    help=f"""
        # File Conversor - CLI

        **{_('Features')}:**

        - {_('Compress image / audio / video / pdf files')}

        - {_('Convert image / audio / video / pdf files')}

        - {_('Batch file processing, for task automation using scripts')}
        
        - {_('Supports various input and output formats')} (mp3, mp4, mkv, jpg, png, webp, pdf, etc)
        
        - {_('Configure default options for conversion / compression')}
        
        - {_('Installs external dependencies automatically (using package managers)')}
    """,
    epilog=f"""
        {_('For more information, visit')} [http://www.github.com/andre-romano/file_conversor](http://www.github.com/andre-romano/file_conversor)
    """)
def main_callback(
        no_log: Annotated[bool, typer.Option(
            "--no-log", "-nl",
            help=_("Disable file logs"),
            is_flag=True,
        )] = False,
        no_progress: Annotated[bool, typer.Option(
            "--no-progress", "-np",
            help=f"{_('Disable progress bars')}",
            is_flag=True,
        )] = False,
        quiet: Annotated[bool, typer.Option(
            "--quiet", "-q",
            help=f"{_('Enable quiet mode (only display errors and progress bars)')}",
            is_flag=True,
        )] = False,
        verbose: Annotated[bool, typer.Option(
            "--verbose", "-v",
            help=_("Enable verbose mode"),
            is_flag=True,
        )] = False,
        debug: Annotated[bool, typer.Option(
            "--debug", "-d",
            help=_("Enable debug mode"),
            is_flag=True,
        )] = False,
):
    STATE.update({
        "no-log": no_log,
        "no-progress": no_progress,
        "quiet": quiet,
        "verbose": verbose,
        "debug": debug,
    })
    logger.debug(f"Command: {sys.argv}")
    State.get_executable()



# src\file_conversor\dependency\__init__.py

"""Module for package managers that provide external dependencies"""

from file_conversor.dependency.abstract_pkg_manager import AbstractPackageManager
from file_conversor.dependency.scoop_pkg_manager import ScoopPackageManager
from file_conversor.dependency.brew_pkg_manager import BrewPackageManager



# src\file_conversor\backend\pillow_backend.py

"""
This module provides functionalities for handling image files using ``pillow`` backend.
"""

from PIL import Image, ImageOps
from PIL.ExifTags import TAGS

from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.abstract_backend import AbstractBackend

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class PillowBackend(AbstractBackend):
    """
    A class that provides an interface for handling image files using ``pillow``.
    """

    Exif = Image.Exif
    Exif_TAGS = TAGS

    SUPPORTED_IN_FORMATS = {
        "bmp": {},
        "gif": {},
        "ico": {},
        "jfif": {},
        "jpg": {},
        "jpeg": {},
        "jpe": {},
        "png": {},
        "psd": {},
        "tif": {},
        "tiff": {},
        "webp": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "bmp": {"format": "BMP"},
        "gif": {"format": "GIF"},
        "ico": {"format": "ICO"},
        "jpg": {"format": "JPEG"},
        "apng": {"format": "PNG"},
        "png": {"format": "PNG"},
        "pdf": {"format": "PDF"},
        "tif": {"format": "TIFF"},
        "webp": {"format": "WEBP"},
    }

    def __init__(self, verbose: bool = False,):
        """
        Initialize the ``pillow`` backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def info(self, input_file: str,) -> Exif:
        """
        Get EXIF info from input file.

        :param input_file: Input image file.

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        img = Image.open(input_file)
        return img.getexif()

    def convert(
        self,
        output_file: str,
        input_file: str,
        quality: int = 90,
        optimize: bool = True,
    ):
        """
        Convert input file into an output.

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param quality: Final quality of image file (1-100). If 100, activates lossless compression. Valid only for JPG, WEBP out formats. Defaults to 90.
        :param optimize: Improve file size, without losing quality (lossless compression). Valid only for JPG, PNG, WEBP out formats Defaults to True.

        :raises ValueError: invalid quality value. Valid values are 1-100.
        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)
        if quality < 1 or quality > 100:
            raise ValueError(f"{_('Invalid quality level. Valid values are')} 1-100.")

        output_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[output_ext]["format"]

        img = Image.open(input_file)
        self._save_fix_errors(
            img,
            output_file,
            format=format,
            quality=quality,
            optimize=optimize,
            lossless=True if quality == 100 else False,  # valid only for WEBP
        )

    def rotate(self, output_file: str, input_file: str, rotate: int):
        """
        Rotate input file by X degrees.

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param rotate: Rotation degrees (0-360).

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = Image.open(input_file)
        img = img.rotate(rotate)
        self._save_fix_errors(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def mirror(self, output_file: str, input_file: str, x_y: bool):
        """
        Mirror input file in relation X or Y axis.

        :param output_file: Output image file.
        :param input_file: Input image file.
        :param x_y: Mirror in relation to x or y axis. True for X axis (mirror image horizontally). False for Y axis (flip image vertically).

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        out_ext = Path(output_file).suffix[1:]
        format = self.SUPPORTED_OUT_FORMATS[out_ext]["format"]

        img = Image.open(input_file)
        if x_y:
            img = ImageOps.mirror(img)
        else:
            img = ImageOps.flip(img)
        self._save_fix_errors(
            img,
            output_file,
            format=format,
            quality=90,
            optimize=True,
        )

    def _save_fix_errors(self, img: Image.Image, output_file: str | Path, format: str, **params):
        """
        Corrects common errors in images and saves them.

        :param img: Image to be corrected.
        :param output_file: File to save img.
        :param format: Target format to save img.
        :param params: Additional parameters for saving the image, such as quality and optimize.

        :raises Exception: if image correction fails.
        """
        try:
            format = format.upper()  # ensure uppercase format

            # 0. Preservar EXIF e ICC se existirem
            if "exif" in img.info and img.info["exif"]:
                params.setdefault("exif", img.info["exif"])
            if "icc_profile" in img.info and img.info["icc_profile"]:
                params.setdefault("icc_profile", img.info["icc_profile"])

            # 1. Transparência -> converter para RGBA
            if img.mode == "P" and "transparency" in img.info:
                img = img.convert("RGBA")

            # 2. Converter modos incompatíveis com o formato de destino
            if format in ("JPEG",) and img.mode not in ("RGB", "L"):
                img = img.convert("RGB")
            elif format in ("PNG", "WEBP") and img.mode not in ("RGB", "RGBA"):
                img = img.convert("RGBA")
            elif format == "TIFF" and img.mode not in ("RGB", "RGBA", "L"):
                img = img.convert("RGB")
            elif format == "BMP" and img.mode not in ("RGB",):
                img = img.convert("RGB")
        except Exception as e:
            logger.error(f"{_('Image correction failed')}: {e}")
            raise

        # save image
        img.save(output_file, format=format, **params)




# src\file_conversor\cli\image_cmd.py

import typer

from pathlib import Path
from typing import Annotated, List

from rich import print
from rich.panel import Panel
from rich.console import Group

# user-provided modules
from file_conversor.backend import PillowBackend, Img2PDFBackend

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.rich import get_progress_bar
from file_conversor.utils.validators import check_file_format, check_is_bool_or_none, check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

image_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = State.get_icons_folder()
    # IMG2PDF commands
    for ext in Img2PDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_pdf",
                description="To PDF",
                command=f'{State.get_executable()} image to-pdf "%1"',
                icon=str(icons_folder_path / "pdf.ico"),
            ),
        ])
    # Pillow commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="info",
                description="Get Info",
                command=f'cmd /k "{State.get_executable()} image info "%1""',
                icon=str(icons_folder_path / "info.ico"),
            ),
            WinContextCommand(
                name="to_jpg",
                description="To JPG",
                command=f'{State.get_executable()} image convert "%1" -o "%1.jpg" -q 95',
                icon=str(icons_folder_path / 'jpg.ico'),
            ),
            WinContextCommand(
                name="to_png",
                description="To PNG",
                command=f'{State.get_executable()} image convert "%1" -o "%1.png" -q 95',
                icon=str(icons_folder_path / 'png.ico'),
            ),
            WinContextCommand(
                name="to_webp",
                description="To WEBP",
                command=f'{State.get_executable()} image convert "%1" -o "%1.webp" -q 95',
                icon=str(icons_folder_path / 'webp.ico'),
            ),
            WinContextCommand(
                name="rotate_anticlock_90",
                description="Rotate Left",
                command=f'{State.get_executable()} image rotate "%1" -r -90',
                icon=str(icons_folder_path / "rotate_left.ico"),
            ),
            WinContextCommand(
                name="rotate_clock_90",
                description="Rotate Right",
                command=f'{State.get_executable()} image rotate "%1" -r 90',
                icon=str(icons_folder_path / "rotate_right.ico"),
            ),
            WinContextCommand(
                name="mirror_x",
                description="Mirror X axis",
                command=f'{State.get_executable()} image mirror "%1" -a x',
                icon=str(icons_folder_path / "left_right.ico"),
            ),
            WinContextCommand(
                name="mirror_y",
                description="Mirror Y axis",
                command=f'{State.get_executable()} image mirror "%1" -a y',
                icon=str(icons_folder_path / "up_down.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


# image info
@image_cmd.command(
    help=f"""
        {_('Get EXIF information about a image file.')}

        {_('This command retrieves metadata and other information about the image file')}:
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor image info filename.webp`

        - `file_conversor image info other_filename.jpg`
    """)
def info(
    filename: Annotated[str, typer.Argument(
        help=f"{_('File')} ({', '.join(PillowBackend.SUPPORTED_IN_FORMATS)})",
        callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_IN_FORMATS, exists=True),
    )],
):
    file_ext = Path(filename).suffix[1:]

    formatted = []
    metadata: PillowBackend.Exif
    with get_progress_bar() as progress:
        task = progress.add_task(f"{_('Parsing file metadata')} ...", total=None)

        pillow_backend = PillowBackend(verbose=STATE['verbose'])
        metadata = pillow_backend.info(filename)
        progress.update(task, total=100, completed=100)

    # 📁 Informações gerais do arquivo
    formatted.append(f"  - {_('Name')}: {filename}")
    formatted.append(f"  - {_('Format')}: {file_ext.upper()}")
    if metadata:
        for tag, value in metadata.items():
            tag_name = PillowBackend.Exif_TAGS.get(tag, f"{tag}")
            formatted.append(f"  - {tag_name}: {value}")

    # Agrupar e exibir tudo com Rich
    group = Group(*formatted)
    print(Panel(group, title=f"🧾 {_('File Analysis')}", border_style="blue"))


# image to-pdf
@image_cmd.command(
    help=f"""
        {_('Convert a list of image files to one PDF file, one image per page.')}

        Fit = {_('Valid only if ``--page-size`` is defined. Otherwise, PDF size = figure size.')}

        - 'into': {_('Figure adjusted to fit in PDF size')}.

        - 'fill': {_('Figure adjusted to fit in PDF size')}, {_('without any empty borders (cut figure if needed)')}.
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor image to-pdf input_file.jpg -o output_file.pdf --dpi 96`

        - `file_conversor image to-pdf input_file1.bmp input_file2.png -o output_file.pdf`

        - `file_conversor image to-pdf input_file.jpg -o output_file.pdf -ps a4_landscape`

        - `file_conversor image to-pdf input_file1.bmp input_file2.png -o output_file.pdf --page-size (21.00,29.70)`
    """)
def to_pdf(
    input_files: Annotated[List[str], typer.Argument(help=f"{_('Input files')} ({', '.join(Img2PDFBackend.SUPPORTED_IN_FORMATS)})",
                                                     callback=lambda x: check_file_format(x, Img2PDFBackend.SUPPORTED_IN_FORMATS, exists=True),
                                                     )],

    output_file: Annotated[str | None, typer.Option("--output", "-o",
                                                    help=f"{_('Output file')} ({', '.join(Img2PDFBackend.SUPPORTED_OUT_FORMATS)}). {_('Defaults to None')} ({_('use the same 1st input file as output name')}).",
                                                    callback=lambda x: check_file_format(x, Img2PDFBackend.SUPPORTED_OUT_FORMATS),
                                                    )] = None,

    dpi: Annotated[int, typer.Option("--dpi", "-d",
                                     help=_("Image quality in dots per inch (DPI). Valid values are between 40-3600."),
                                     min=40, max=3600,
                                     )] = CONFIG["image-dpi"],

    fit: Annotated[str, typer.Option("--fit", "-f",
                                     help=_("Image fit. Valid only if ``--page-size`` is defined. Valid values are 'into', or 'fill'. Defaults to 'into'. "),
                                     callback=lambda x: check_valid_options(x.lower(), ['into', 'fill']),
                                     )] = CONFIG["image-fit"],

    page_size: Annotated[str | None, typer.Option("--page-size", "-ps",
                                                  help=_("Page size. Format '(width, height)'. Other valid values are: 'a4', 'a4_landscape'. Defaults to None (PDF size = image size)."),
                                                  callback=lambda x: check_valid_options(x.lower() if x else None, [None, 'a4', 'a4_landscape']),
                                                  )] = CONFIG["image-page-size"],

    set_metadata: Annotated[bool, typer.Option("--set-metadata", "-sm",
                                               help=_("Set PDF metadata. Defaults to True (set creator, producer, modification date, etc)."),
                                               callback=check_is_bool_or_none,
                                               is_flag=True,
                                               )] = CONFIG["image-set-metadata"],
):
    img2pdf_backend = Img2PDFBackend(verbose=STATE['verbose'])
    # display current progress
    with get_progress_bar() as progress:
        task = progress.add_task(f"{_('Processing file')} '{output_file}':", total=None)

        # parse user input
        image_fit: Img2PDFBackend.FIT_MODE
        if fit == 'into':
            image_fit = Img2PDFBackend.FIT_INTO
        elif fit == 'fill':
            image_fit = Img2PDFBackend.FIT_FILL

        page_sz: tuple | None
        if page_size is None:
            page_sz = Img2PDFBackend.LAYOUT_NONE
        elif page_size == 'a4':
            page_sz = Img2PDFBackend.LAYOUT_A4_PORTRAIT_CM
        elif page_size == 'a4_landscape':
            page_sz = Img2PDFBackend.LAYOUT_A4_LANDSCAPE_CM
        else:
            page_sz = tuple(page_size)

        img2pdf_backend.to_pdf(
            input_files=input_files,
            output_file=output_file if output_file else f"{input_files[0].replace(".pdf", "")}.pdf",
            dpi=dpi,
            image_fit=image_fit,
            page_size=page_sz,
            include_metadata=set_metadata,
        )
        progress.update(task, total=100, completed=100)
    logger.info(f"{_('PDF generation')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


# image convert
@image_cmd.command(
    help=f"""
        {_('Convert a image file to a different format.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor image convert input_file.webp -o output_file.jpg --quality 85`

        - `file_conversor image convert input_file.bmp -o output_file.png`
    """)
def convert(
    input_file: Annotated[str, typer.Argument(
        help=f"{_('Input file')} ({', '.join(PillowBackend.SUPPORTED_IN_FORMATS)})",
        callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_IN_FORMATS, exists=True),
    )],

    output_file: Annotated[str, typer.Option("--output", "-o",
                                             help=f"{_('Output file')} ({', '.join(PillowBackend.SUPPORTED_OUT_FORMATS)})",
                                             callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_OUT_FORMATS),
                                             )],

    quality: Annotated[int, typer.Option("--quality", "-q",
                                         help=_("Image quality. Valid values are between 1-100."),
                                         min=1, max=100,
                                         )] = CONFIG["image-quality"],
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])
    # display current progress
    with get_progress_bar() as progress:
        task = progress.add_task(f"{_('Processing file')} '{input_file}':", total=None)
        pillow_backend.convert(
            input_file=input_file,
            output_file=output_file,
            quality=quality,
        )
        progress.update(task, total=100, completed=100)
    logger.info(f"{_('Image convertion')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


# image rotate
@image_cmd.command(
    help=f"""
        {_('Rotate a image file (clockwise or anti-clockwise).')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor pdf rotate input_file.jpg -o output_file.jpg -r 90` 
        
        - `file_conversor pdf rotate input_file.jpg -o output_file.jpg -r -180` 
    """)
def rotate(
    input_file: Annotated[str, typer.Argument(
        help=f"{_('Input file')} ({', '.join(PillowBackend.SUPPORTED_IN_FORMATS)})",
        callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_IN_FORMATS, exists=True),
    )],

    rotation: Annotated[int, typer.Option("--rotation", "-r",
                                          help=_("Rotation in degrees. Valid values are between -360 (anti-clockwise rotation) and 360 (clockwise rotation)."),
                                          min=-360, max=360,
                                          )],
    output_file: Annotated[str | None, typer.Option("--output", "-o",
                                                    help=f"{_('Output file')} ({', '.join(PillowBackend.SUPPORTED_OUT_FORMATS)}). {_('Defaults to None')} ({_('use the same input file as output name')}, {_('with _rotated at the end)')}",
                                                    callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_OUT_FORMATS),
                                                    )] = None,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])
    # display current progress
    with get_progress_bar() as progress:
        input_path = Path(input_file)
        input_name = input_path.with_suffix("").name
        input_ext = input_path.suffix[1:]

        task = progress.add_task(f"{_('Processing file')} '{input_file}':", total=None)
        pillow_backend.rotate(
            input_file=input_file,
            output_file=output_file if output_file else f"{input_name}_rotated.{input_ext}",
            rotate=rotation,
        )
        progress.update(task, total=100, completed=100)
    logger.info(f"{_('Image rotation')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


# image mirror
@image_cmd.command(
    help=f"""
        {_('Mirror an image file (vertically or horizontally).')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor pdf mirror input_file.jpg -o output_file.jpg -a x` 
        
        - `file_conversor pdf mirror input_file.jpg -o output_file.jpg -a y` 
    """)
def mirror(
    input_file: Annotated[str, typer.Argument(
        help=f"{_('Input file')} ({', '.join(PillowBackend.SUPPORTED_IN_FORMATS)})",
        callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_IN_FORMATS, exists=True),
    )],

    axis: Annotated[str, typer.Option("--axis", "-a",
                                      help=_("Axis. Valid values are 'x' (mirror horizontally) or 'y' (flip vertically)."),
                                      callback=lambda x: check_valid_options(x, valid_options=['x', 'y']),
                                      )],
    output_file: Annotated[str | None, typer.Option("--output", "-o",
                                                    help=f"{_('Output file')} ({', '.join(PillowBackend.SUPPORTED_OUT_FORMATS)}). {_('Defaults to None')} ({_('use the same input file as output name')}, {_('with _mirrored at the end)')}",
                                                    callback=lambda x: check_file_format(x, PillowBackend.SUPPORTED_OUT_FORMATS),
                                                    )] = None,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])
    # display current progress
    with get_progress_bar() as progress:
        input_path = Path(input_file)
        input_name = input_path.with_suffix("").name
        input_ext = input_path.suffix[1:]

        task = progress.add_task(f"{_('Processing file')} '{input_file}':", total=None)
        pillow_backend.mirror(
            input_file=input_file,
            output_file=output_file if output_file else f"{input_name}_mirrored.{input_ext}",
            x_y=True if axis == "x" else False,
        )
        progress.update(task, total=100, completed=100)
    logger.info(f"{_('Image mirroring')}: [green][bold]{_('SUCCESS')}[/bold][/green]")



# src\file_conversor\backend\pdf\__init__.py

from file_conversor.backend.pdf.pikepdf_backend import PikePDFBackend
from file_conversor.backend.pdf.pymupdf_backend import PyMuPDFBackend
from file_conversor.backend.pdf.pypdf_backend import PyPDFBackend
from file_conversor.backend.pdf.ghostscript_backend import GhostscriptBackend



# src\file_conversor\backend\office\powerpoint_backend.py

from pathlib import Path

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.office.abstract_msoffice_backend import AbstractMSOfficeBackend, Win32Com

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class PowerPointBackend(AbstractMSOfficeBackend):
    """
    A class that provides an interface for handling doc files using ``powerpoint`` (comtypes).
    """

    SUPPORTED_IN_FORMATS = {
        "ppt": {},
        "pptx": {},
        "odp": {},
    }
    SUPPORTED_OUT_FORMATS = {
        # format = wdFormat VBA code
        # https://learn.microsoft.com/en-us/office/vba/api/powerpoint.ppsaveasfiletype
        "ppt": {'format': 1},
        "pptx": {'format': 24},
        "odp": {'format': 35},
        "pdf": {'format': 32},
    }

    def __init__(
        self,
        install_deps: bool | None = None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Reserved for future use. Defaults to None.
        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(
            prog_id="PowerPoint.Application",
            install_deps=install_deps,
            verbose=verbose,
        )

    def convert(
        self,
        output_file: str,
        input_file: str,
    ):
        input_path = Path(input_file).resolve()
        output_path = Path(output_file).resolve()

        self.check_file_exists(str(input_path))

        out_config = PowerPointBackend.SUPPORTED_OUT_FORMATS[output_path.suffix[1:]]

        # powerpoint.Visible = True  # needed for powerpoint
        with Win32Com(self.PROG_ID, visible=None) as powerpoint:
            presentation = powerpoint.Presentations.Open(str(input_path), WithWindow=False)
            presentation.SaveAs(
                str(output_path),
                FileFormat=out_config['format'],
            )
            presentation.Close()



# src\file_conversor\backend\ffmpeg_backend.py

"""
This module provides functionalities for handling audio and video files using FFmpeg.
"""

import json
import subprocess
import re

from pathlib import Path
from datetime import timedelta
from typing import Iterable

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.validators import check_file_format

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager
from file_conversor.backend.abstract_backend import AbstractBackend

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class FFmpegBackend(AbstractBackend):
    """
    FFmpegBackend is a class that provides an interface for handling audio and video files using FFmpeg.
    """

    SUPPORTED_IN_AUDIO_FORMATS = {
        'aac': {},
        'ac3': {},
        'flac': {},
        'm4a': {},
        'mp3': {},
        'ogg': {},
        'opus': {},
        'wav': {},
    }
    SUPPORTED_IN_VIDEO_FORMATS = {
        '3gp': {},
        'asf': {},
        'avi': {},
        'flv': {},
        'h264': {},
        'hevc': {},
        'm4v': {},
        'mkv': {},
        'mov': {},
        'mp4': {},
        'mpeg': {},
        'mpg': {},
        'webm': {},
    }
    SUPPORTED_IN_FORMATS = SUPPORTED_IN_AUDIO_FORMATS | SUPPORTED_IN_VIDEO_FORMATS

    SUPPORTED_OUT_AUDIO_FORMATS = {
        'mp3': {
            '-f': 'mp3',
            '-c:a': 'libmp3lame',
        },
        'm4a': {
            '-f': 'ipod',
            '-c:a': 'aac',
        },
        'ogg': {
            '-f': 'ogg',
            '-c:a': 'libvorbis',
        },
        'opus': {
            '-f': 'opus',
            '-c:a': 'libopus',
        },
        'flac': {
            '-f': 'flac',
            '-c:a': 'flac',
        },
    }
    SUPPORTED_OUT_VIDEO_FORMATS = {
        'mp4': {
            '-f': 'mp4',
            '-c:v': 'libx264',
            '-c:a': 'aac',
        },
        'avi': {
            '-f': 'avi',
            '-c:v': 'mpeg4',
            '-c:a': 'libmp3lame',
        },
        'mkv': {
            '-f': 'matroska',
            '-c:v': 'libx264',
            '-c:a': 'aac',
        },
        'webm': {
            '-f': 'webm',
            '-c:v': 'libvpx',
            '-c:a': 'libvorbis',
        },
    }
    SUPPORTED_OUT_FORMATS = SUPPORTED_OUT_VIDEO_FORMATS | SUPPORTED_OUT_AUDIO_FORMATS

    PROGRESS_RE = re.compile(r'time=(\d+):(\d+):([\d\.]+)')

    @staticmethod
    def get_convert_progress(process: subprocess.Popen, file_duration_secs: float) -> float:
        """
        Gets FFMpeg current progress status, in percentage (0-100)

        :param process: Subprocess.Popen object.
        :param file_duration_secs: Input file duration (in secs).

        :return: Progress of FFMpeg [0-100], otherwise 0
        """
        if process and process.stderr:
            match = FFmpegBackend.PROGRESS_RE.search(process.stderr.readline())
            if match:
                hours = int(match.group(1))
                minutes = int(match.group(2))
                seconds = float(match.group(3))
                current_time = hours * 3600 + minutes * 60 + seconds
                return 100.0 * (float(current_time) / file_duration_secs)
        return 0

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the FFMpeg backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if ffmpeg dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "ffmpeg": "ffmpeg"
                }),
                BrewPackageManager({
                    "ffmpeg": "ffmpeg"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        # check ffprobe / ffmpeg
        self._ffprobe_bin = self.find_in_path("ffprobe")
        self._ffmpeg_bin = self.find_in_path("ffmpeg")

    def calculate_file_total_duration(self, file_path: str) -> float:
        """
        Calculate file total duration (in secs), using `ffprobe`.

        :return: Total duration in seconds.
        """
        result = subprocess.run(
            [str(self._ffprobe_bin), '-v', 'error', '-show_entries',
             'format=duration', '-of',
             'default=noprint_wrappers=1:nokey=1', file_path],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True
        )
        duration_str = result.stdout.strip()
        return float(duration_str if duration_str else "0")

    def calculate_file_formatted_duration(self, file_path: str) -> str:
        """
        Calculate file duration (formatted), using `ffprobe`.

        :return: Total duration  (format HH:MM:SS).
        """
        duration_secs = self.calculate_file_total_duration(file_path)

        # Converte segundos para timedelta e formata como HH:MM:SS
        td = timedelta(seconds=int(duration_secs))
        return str(td)

    def get_file_info(self, file_path: str) -> dict:
        """
        Executa ffprobe e retorna os metadados no formato JSON

        result = {
            streams: [],
            chapters: [],
            format: {},
        }

        stream = {
            index,
            codec_name,
            codec_long_name,
            codec_type: audio|video,
            sampling_rate,
            channels,
            channel_layout: stereo|mono,
        }

        format = {
            format_name,
            format_long_name,
            duration,
            size,
        }

        :return: JSON object
        """
        command = [
            str(self._ffprobe_bin),
            "-v", "quiet",
            "-print_format", "json",
            "-show_format",
            "-show_streams",
            "-show_chapters",
            "-show_error",
            file_path
        ]
        result = subprocess.run(
            command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        return json.loads(result.stdout)

    def _set_input(self, input_file: str) -> tuple[str, list]:
        """
        Set the input file and check if it has a supported format.

        :param input_file: Input file path.

        :return: (Input file, in options).

        :raises FileNotFoundError: If the input file does not exist.
        :raises ValueError: If the input file format is not supported.
        """
        # check file is found
        input_path = Path(input_file)
        if not input_path.exists() and not input_path.is_file():
            raise FileNotFoundError(f"Input file '{input_file}' not found")

        # check if the input file has a supported format
        check_file_format(input_path, self.SUPPORTED_IN_FORMATS)

        # set the input format options based on the file extension
        in_opts = []
        in_ext = input_path.suffix[1:]
        for opt, value in self.SUPPORTED_IN_FORMATS[in_ext].items():
            in_opts.extend([opt, value])

        return input_file, in_opts

    def _set_output(self, output_file: str) -> tuple[str, list]:
        """
        Set the output file and check if it has a supported format.

        :param output_file: Output file path.

        :return: (Output file, out options).

        :raises typer.BadParameter: Unsupported format, or file not found.
        """
        output_path = Path(output_file)

        # create out dir (if it does not exists)
        output_path.parent.mkdir(exist_ok=True)

        # check if the output file has a supported format
        check_file_format(output_path, self.SUPPORTED_OUT_FORMATS)

        # set the output format options based on the file extension
        out_opts = []
        out_ext = output_path.suffix[1:]
        for opt, value in self.SUPPORTED_OUT_FORMATS[out_ext].items():
            out_opts.extend([opt, value])

        return output_file, out_opts

    def convert(
        self,
            input_file: str,
            output_file: str,
            overwrite_output: bool = True,
            stats: bool = False,
            in_options: Iterable | None = None,
            out_options: Iterable | None = None,
    ) -> subprocess.Popen:
        """
        Execute the FFmpeg command to convert the input file to the output file.

        :param input_file: Input file path.
        :param output_file: Output file path.      
        :param overwrite_output: Overwrite output file (no user confirmation prompt). Defaults to True.      
        :param stats: Show progress stats. Defaults to False.      
        :param in_options: Additional input options. Defaults to None.      
        :param out_options: Additional output options. Defaults to None.    

        :return: Subprocess.Popen object

        :raises RuntimeError: If FFmpeg encounters an error during execution.
        """
        # set input/output files and options
        in_file, in_opts = self._set_input(input_file)
        out_file, out_opts = self._set_output(output_file)

        in_opts.extend(in_options if in_options else [])
        out_opts.extend(out_options if out_options else [])

        # set global options
        global_options = [
            # overwrite output (no confirm)
            "-y" if overwrite_output else "-n",
            "-v" if self._verbose else "",  # verbose output
            "-stats" if stats else "",  # print progress stats
        ]

        # build ffmpeg command
        ffmpeg_command = []
        ffmpeg_command.extend([str(self._ffmpeg_bin)])  # ffmpeg CLI
        ffmpeg_command.extend(global_options)    # set global options
        ffmpeg_command.extend(in_opts)           # set in options
        ffmpeg_command.extend(["-i", in_file])   # set input
        ffmpeg_command.extend(out_opts)          # set out options
        ffmpeg_command.extend([out_file])        # set output

        # remove empty strings
        ffmpeg_command = [arg for arg in ffmpeg_command if arg != ""]

        logger.info(f"Executing FFmpeg ...")
        logger.debug(f"{" ".join(ffmpeg_command)}")

        # Execute the FFmpeg command
        _convert_process = subprocess.Popen(
            ffmpeg_command,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True
        )
        return _convert_process
